/*
 *  This file is part of M-Index library.
 *
 *  M-Index library is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  M-Index library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with M-Index library.  If not, see <http://www.gnu.org/licenses/>.
 */
package mindex;

import java.util.Properties;
import messif.utility.ExtendedProperties;
import messif.utility.ExtendedPropertiesException;

/**
 * This class extends the {@link ExtendedProperties} by identification of the
 * properties by enriched keys of class {@link Parameter}. The class is used for
 * M-Index configuration.
 *
 * @author David Novak, Masaryk University, Brno, Czech Republic, david.novak@fi.muni.cz
 */
public class MIndexProperties extends ExtendedProperties {

    /**
     * Creates these properties from any other properties, a (potential empty)
     * prefix, and (potentially null) default properties.
     *
     * @param properties any properties to create these enriched properties from
     * @param prefix prefix to restrict the passed properties
     * @param defaults defaults (can be null)
     */
    public MIndexProperties(Properties properties, String prefix) {
        super();
        load(properties, prefix, null);
    }

    
    // ***************************      Getters and setters for {@link Parameter}     ******************* //
    
    public int getIntProperty(Parameter key) throws ExtendedPropertiesException {
        if (key.getType() != Parameter.Type.INT) {
            throw new ExtendedPropertiesException("Property " + key.getKey() + " is not of type Integer");
        }
        return super.getIntProperty(key.getKey(), Integer.valueOf(key.getDefaultValue()));
    }

    public short getShortProperty(Parameter key) throws ExtendedPropertiesException {
        if (key.getType() != Parameter.Type.SHORT) {
            throw new ExtendedPropertiesException("Property " + key.getKey() + " is not of type Short");
        }
        return (short) super.getIntProperty(key.getKey(), Integer.valueOf(key.getDefaultValue()));
    }

    public long getLongProperty(Parameter key) throws ExtendedPropertiesException {
        if (key.getType() != Parameter.Type.LONG) {
            throw new ExtendedPropertiesException("Property " + key.getKey() + " is not of type Long");
        }
        return Long.valueOf(getProperty(key.getKey(), key.getDefaultValue()));
    }

    public boolean getBoolProperty(Parameter key) throws ExtendedPropertiesException {
        if (key.getType() != Parameter.Type.BOOL) {
            throw new ExtendedPropertiesException("Property " + key.getKey() + " is not of type Boolean");
        }
        return super.getBoolProperty(key.getKey(), Boolean.valueOf(key.getDefaultValue()));
    }
    
    public String getProperty(Parameter key) throws ExtendedPropertiesException {
        return super.getProperty(key.getKey(), key.getDefaultValue());
    }

    public void setProperty(Parameter key, String value) {
        super.setProperty(key.getKey(), value);
    }

    public void setProperty(Parameter key, boolean value) {
        if (key.getType() != Parameter.Type.BOOL) {
            throw new ExtendedPropertiesException("Property " + key.getKey() + " is not of type Boolean");
        }
        super.setProperty(key.getKey(), Boolean.toString(value));
    }

    public void setProperty(Parameter key, int value) {
        if (key.getType() != Parameter.Type.INT) {
            throw new ExtendedPropertiesException("Property " + key.getKey() + " is not of type Integer");
        }
        super.setProperty(key.getKey(), Integer.toString(value));
    }

    public void setProperty(Parameter key, short value) {
        if (key.getType() != Parameter.Type.SHORT) {
            throw new ExtendedPropertiesException("Property " + key.getKey() + " is not of type Short");
        }
        super.setProperty(key.getKey(), Short.toString(value));
    }
    
}
