package mindex.distance;

/**
 * Extension of the {@link QueryPPPDistanceCalculator} that enables to calculate only
 *  partial value of the distance starting from given level.
 * @author David Novak
 */
public interface PartialQueryPPPDistanceCalculator extends QueryPPPDistanceCalculator {
    
    /**
     * Calculate partial distance between a query (typically represented by full PP or query-pivot
     *  distances) and given pivot permutation prefix. Skip the [1, fromLevel) levels from the calculation.
     * @param ppp pivot permutation prefix of some length
     * @param fromLevel the level from which this partial distance should be calculated
     * @return partial distance between query and given PPP
     */
    public float getPartialQueryDistance(final short [] ppp, final int fromLevel);
            
    /**
     * Calculate partial distance between a query (typically represented by full PP or query-pivot
     *  distances) and given pivot permutation prefix. Skip the [1, fromLevel) levels from the calculation.
     * @param ppIndex pivot permutation item at level {@code fromLevel}
     * @param fromLevel the level from which this partial distance should be calculated
     * @return partial distance between query and given PPP
     */
    public float getPartialQueryDistance(final short ppIndex, final int fromLevel);
    
}
