/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package mindex.distance;

import messif.algorithms.AlgorithmMethodException;
import messif.operations.RankingSingleQueryOperation;
import mindex.MetricIndex;


public class WeightedSumDistCalculatorPure extends QueryPivotDistDiffCalculator implements PartialQueryPPPDistanceCalculator {

    public static final float DEFAULT_COEF = 0.8f;
    public static final String WEIGHTED_SUM_COEF_PARAM = "WEIGHTED_SUM_COEF_PARAM";
    
    private float [] COEFS;
    private float [] COEF_SUMS;
    
    public WeightedSumDistCalculatorPure(RankingSingleQueryOperation operation, MetricIndex mIndex) throws AlgorithmMethodException {
        super(operation.getQueryObject(), mIndex);
        initCoeficients(operation.getParameter(WEIGHTED_SUM_COEF_PARAM, Float.class, DEFAULT_COEF));
    }
    
    protected WeightedSumDistCalculatorPure(short maxLevel, float [] queryPivotDistances, short [] queryPPP) {
        super(maxLevel, queryPivotDistances, queryPPP);
        initCoeficients(DEFAULT_COEF);
    }
    
    private void initCoeficients(float coeficient) {
        COEFS = new float[maxLevel];
        COEF_SUMS = new float[maxLevel];
        COEFS[0] = 1f;
        for (int i = 1; i < maxLevel; i++) {
            COEFS[i] = COEFS[i-1] * coeficient;
            COEF_SUMS[i] = COEF_SUMS[i-1] + COEFS[i-1];
        }
    }

    // **************   Distance calculations    ************************** //
    
    @Override
    public final float getQueryDistance(final short[] ppp) {
        return getDistance(ppp, 1, getMaxLevel(ppp.length));
    }

    @Override
    public float getPartialQueryDistance(final short[] ppp, final int fromLevel) {
        if (ppp.length == 0) {
            return 0f;
        }
        return getDistance(ppp, fromLevel, getMaxLevel(fromLevel - 1 + ppp.length));
    }

    private float getDistance(final short[] ppp, final int fromLevel, final int toLevel) {
        float sum = 0f;
        for (int i = 0; i <= toLevel - fromLevel; i++) {
            sum += (queryPivotDistances[ppp[i]] * COEFS[i + fromLevel - 1]);
        }
        return sum;
    }

    @Override
    public float getPartialQueryDistance(short ppIndex, int fromLevel) {
        return (queryPivotDistances[ppIndex] * COEFS[fromLevel - 1]);
    }
    
    
    public final float getCellDistanceEstimation(short[] ppp) {        
        float sum = 0f;
        for (int i = 0; i < getMaxLevel(ppp.length); i++) {
            sum += (queryPivotDistances[ppp[i]] * COEFS[i]);
        }
        
        return (ppp.length == 0) ? sum : sum / COEF_SUMS[getMaxLevel(ppp.length)];
    }

}
