/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package mindex.distance;

import messif.algorithms.AlgorithmMethodException;
import messif.operations.RankingSingleQueryOperation;
import mindex.MetricIndex;
import mindex.navigation.VoronoiInternalCell;


public class WeightedSumDistCalculatorTight extends QueryPivotDistDiffCalculator {

    public static final float DEFAULT_COEF = 0.8f;
    public static final String WEIGHTED_SUM_COEF_PARAM = "WEIGHTED_SUM_COEF_PARAM";
    
    private float [] COEFS;

    public WeightedSumDistCalculatorTight(RankingSingleQueryOperation operation, MetricIndex mIndex) throws AlgorithmMethodException {
        super(operation.getQueryObject(), mIndex);
        initCoeficients(operation.getParameter(WEIGHTED_SUM_COEF_PARAM, Float.class, DEFAULT_COEF));
    }
    
    protected WeightedSumDistCalculatorTight(short maxLevel, float [] queryPivotDistances, short [] queryPPP) {
        super(maxLevel, queryPivotDistances, queryPPP);
        initCoeficients(DEFAULT_COEF);
    }
    
    private void initCoeficients(float coeficient) {
        COEFS = new float[maxLevel];
        COEFS[0] = 1f;
        for (int i = 1; i < COEFS.length; i++) {
            COEFS[i] = COEFS[i-1] * coeficient;
        }
    }

    // **************   Distance calculations    ************************** //

    @Override
    public final float getQueryDistance(final short[] ppp) {
        return getQueryDistance(ppp, null);
    }

    @Override
    public final float getQueryDistance(final VoronoiInternalCell intCell) {
        return getQueryDistance(intCell.getPppForReading(), intCell);
    }
    
    protected final float getQueryDistance(final short[] ppp, VoronoiInternalCell intCell) {
        int length = getMaxLevel(ppp.length);
        float sum = 0f;
        for (int i = 0; i < length; i++) {
            sum += (queryPivotDistances[ppp[i]] * COEFS[i]);
        }
        
        // correction to estimate the minimum possible distance of data with this ppp prefix
        int smallestNonusedIndex = 0;
        int usedFirstIndex = -1;
        for (int i = length; i < maxLevel; i++) {
            if (smallestNonusedIndex == usedFirstIndex) {
                smallestNonusedIndex++;
            }

            int j = 0;
            while (j < length) {
                if (queryPP[smallestNonusedIndex] == ppp[j++]) {
                    if (++smallestNonusedIndex == usedFirstIndex) {
                        smallestNonusedIndex++;
                    }
                    j = 0;
                }
            }

            // check the really existing nodes on the next level
            if (i == length && intCell != null) {
                usedFirstIndex = smallestNonusedIndex;
                while (intCell.getChildNode(queryPP[usedFirstIndex]) == null) {
                    usedFirstIndex++;
                }
                sum += queryPivotDistances[queryPP[usedFirstIndex]] * COEFS[i];
            } else {
                sum += queryPivotDistances[queryPP[smallestNonusedIndex++]] * COEFS[i];
            }
        }
        return sum;
    }
    
}
