package mindex.navigation;

import java.util.List;
import messif.objects.LocalAbstractObject;

/**
 * An auxiliary class encapsulating if agile multibucket partitioning should be 
 * used during a split + number of objects that was really inserted into the bucket by the
 * operation that caused the split/
 * @author Ondrej Nevelik, Masaryk University, Brno, Czech Republic, nevelik@gmail.com
 */
public class SplitConfiguration {
    
    /** Number of objects that was really inserted by the last insert operation */
    private int numberOfInsertedObjects = 0;
    
    /** Flag determining if agile multi bucket partitioning should be used */
    private boolean agileMultibucketPartitioning;
    
    /** List of objects to be reinserted after the split */
    private List<? extends LocalAbstractObject> toReinsert;
    
    public SplitConfiguration(boolean agileMultibucketPartitioning, List<LocalAbstractObject> toReinsert) {
        this(agileMultibucketPartitioning, 0, toReinsert);
    }
        
    public SplitConfiguration(boolean agileMultibucketPartitioning, int numberOfInsertedObjects, List<? extends LocalAbstractObject> toReinsert) {
        this.agileMultibucketPartitioning = agileMultibucketPartitioning;
        this.numberOfInsertedObjects = numberOfInsertedObjects;
        this.toReinsert = toReinsert;
    }
    
    public boolean isAgileMultibucketPartitioning() {
        return agileMultibucketPartitioning;
    }
    
    public int getNumberOfInsertedObjects() {
    	return numberOfInsertedObjects;
    }

    public List<? extends LocalAbstractObject> getToReinsert() {
        return toReinsert;
    }        
    
}
